/*
 * a12nd.f.c
 * Copyright (C) 2008-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <a1-inter.h>
#include <a1-types.h>
#include <af.h>
#include <e4.h>
#include <lxcast.h>

#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

static int slip_cast(struct x1f4_dxcast_type *, struct a1_shuffle_type *);
static int slip_miss(struct x1f4_dxcast_type *, struct a1_shuffle_type *);
static int slip_node(struct x1f4_dxcast_type *, struct a1_shuffle_type *);
static int slip_text(struct x1f4_dxcast_type *, struct a1_shuffle_type *);

static int
slip_cast(struct x1f4_dxcast_type *dxcast_data,
	  struct a1_shuffle_type *shuffle_data)
{
    int status, type;

    type = dxcast_data->type;
    if (type < X1f4_E4_LAST) {
	if (type == X1f4_E4_TEXT) {
	    status = slip_text(dxcast_data, shuffle_data);
	} else {
	    status = 0;
	}
    } else {
	if (X1f4_E4_CALL < type) {
	    status = _x1f4_slip_effect(NULL, (void *) &dxcast_data->data);
	} else {
	    status = slip_node(dxcast_data, shuffle_data);
	}
    }

    return status;
}


static int
slip_miss(struct x1f4_dxcast_type *dxcast_data,
	  struct a1_shuffle_type *shuffle_data)
{
    const char *name;
    int status;
    const struct x1f4_datatype_type *datatype_data;

    datatype_data = shuffle_data->u.sidetype_data;
    name = datatype_data->name;
    while (name) {
	if (datatype_data->type == X1f4_E4_CASE) {
	    break;
	} else {
	    datatype_data++;
	    name = datatype_data->name;
	}
    }
    if (name) {
    } else {
	datatype_data = shuffle_data->u.datatype_data;
	name = datatype_data->name;
	while (name) {
	    if (datatype_data->type == X1f4_E4_CASE) {
		break;
	    } else {
		datatype_data++;
		name = datatype_data->name;
	    }
	}
    }

    if (name) {
	if (datatype_data->slip) {
	    status = datatype_data->slip
		(datatype_data->context, (void *) &dxcast_data);
	} else {
	    status = 0;
	}
    } else {
	status = 0;
    }

    return status;
}


static int
slip_node(struct x1f4_dxcast_type *dxcast_data,
	  struct a1_shuffle_type *shuffle_data)
{
    const char *name;
    int status, type;
    const struct x1f4_datatype_type *datatype_data;

    type = dxcast_data->type;

    datatype_data = shuffle_data->u.datatype_data;
    name = datatype_data->name;
    while (name) {
	if (datatype_data->type == type) {
	    break;
	} else {
	    datatype_data++;
	    name = datatype_data->name;
	}
    }
    if (name) {
    } else {
	datatype_data = shuffle_data->u.sidetype_data;
	name = datatype_data->name;
	while (name) {
	    if (datatype_data->type == type) {
		break;
	    } else {
		datatype_data++;
		name = datatype_data->name;
	    }
	}
    }

    if (name) {
	if (datatype_data->slip) {
	    status = datatype_data->slip
		(datatype_data->context, (void *) &dxcast_data->data);
	} else {
	    status = 0;
	}
    } else {
	status = 0;
    }

    return status;
}


static int
slip_text(struct x1f4_dxcast_type *dxcast_data,
	  struct a1_shuffle_type *shuffle_data)
{
    int status;

    status = shuffle_data->m.free
	(shuffle_data->m.data, I_TEXT(&dxcast_data->data));
    if (status) {
	status = _x1f4_a1_stat_free(shuffle_data);
    }

    return status;
}


int
_x1f4_a1_slip_side(struct a1_shuffle_type *shuffle_data, unsigned prefix,
		   unsigned offset, void *line,
		   const struct x1f4_function_type *function_data)
{
    const int *args;
    int excess, status = 0;
    struct x1f4_dxcast_type *dxcast_data;
    unsigned count, flags, i;

    dxcast_data = line;

    count = function_data->count;

    flags = function_data->flags;

    args = function_data->args + prefix;

    for (i = offset; i; i--) {
	int type;

	type = *args;
	if (type == X1f4_E4_CASE) {
	    status = slip_miss(dxcast_data, shuffle_data);
	    if (status) {
		break;
	    }
	} else {
	    do {
		dxcast_data->type = type;

		if (flags & X1f4_E4_POST_TYPE) {
		    if (args[count] & X1f4_E4_POST_XSET) {
			if (1) {
			    break;
			}
		    }
		}

		excess = slip_cast(dxcast_data, shuffle_data);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    } while (0);
	}

	args++;

	dxcast_data++;
    }

    excess = shuffle_data->m.free(shuffle_data->m.data, line);
    if (excess) {
	if (status) {
	} else {
	    status = _x1f4_a1_stat_free(shuffle_data);
	}
    }

    return status;
}
