/*
 * lxdate.h
 * Copyright (C) 2008-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxdate_H__
#define __LIBx1f4_lxdate_H__

#include <time.h>

#include <dt.h>
#include <nl.h>
#include <trans.h>

/*
 * error definitions
 */
#define X1f4_LXDATE_FREE_ERROR		-2
#define X1f4_LXDATE_LINK_ERROR		-3
#define X1f4_LXDATE_MODE_ERROR		-4

/*
 * generation flag definitions
 */
#define X1f4_LXDATE_AUTOLINK		(1L << 0)
#define X1f4_LXDATE_CODELINK		(1L << 1)
#define X1f4_LXDATE_EVERLINK		(1L << 2)
#define X1f4_LXDATE_RESOURCE		(1L << 3)
#define X1f4_LXDATE_TEXTFLAT		(1L << 4)

#if !defined __x1f4_function_type_defined__
typedef struct x1f4_function_type {
    const char *name;
    int (*function) (void *, void *, void **), type;
    const int *args;
    unsigned count, flags, length;
} x1f4_function_type;
#endif				/* !__x1f4_function_type_defined__ */

#if !defined __x1f4_function_type_defined__
# define __x1f4_function_type_defined__
#endif				/* !__x1f4_function_type_defined__ */

#if !defined __x1f4_linetext_type_defined__
typedef struct x1f4_linetext_type {
    struct x1f4_function_type function;
    const void *context;
} x1f4_linetext_type;
#endif				/* !__x1f4_linetext_type_defined__ */

#if !defined __x1f4_linetext_type_defined__
# define __x1f4_linetext_type_defined__
#endif				/* !__x1f4_linetext_type_defined__ */

typedef struct x1f4_dxdate_type {
    time_t time;
    unsigned call, second_f, second_i, second_l;
} x1f4_dxdate_type;

typedef struct x1f4_dxtime_type {
    unsigned call, second_a, second_f, second_l;
} x1f4_dxtime_type;

typedef struct x1f4_lxdate_type {
    struct {
	int (*line) (void *), (*post) (void *),
	    (*push) (void *, const char *, unsigned);
	void *data;
    } textflat_set;
    struct {
	int type;
    } datetype_set;
    struct {
	int type;
    } timetype_set;
    struct {
	int (*free) (void *, void *), (*link) (void *, void **, unsigned),
	    (*mode) (void *, void **, unsigned);
	void *data;
    } resource_set;
    struct x1f4_trans_type codelink_set;
} x1f4_lxdate_type;

extern int x1f4_copy_lxdate
    (void *, void *, void **, const void *,
     const struct x1f4_nodelink_type *);
extern int x1f4_copy_lxtime
    (void *, void *, void **, const void *,
     const struct x1f4_nodelink_type *);
extern int x1f4_dset_lxdate(void *, const struct x1f4_datatype_type **);
extern int x1f4_even_lxtime(void *, void *, void *, const void *);
extern int x1f4_fare_lxtime
    (void *, void *, void **, const void *, const struct x1f4_nodelink_type *);
extern int x1f4_fini_lxdate(void **);
extern int x1f4_flat_lxdate(void *, void **);
extern int x1f4_flat_lxtime(void *, void **);
extern int x1f4_fset_lxdate(void *, const struct x1f4_linetext_type **);
extern int x1f4_init_lxdate
    (void **, unsigned, const struct x1f4_lxdate_type *);
extern int x1f4_lead_lxtime(void *, void **);
extern int x1f4_line_lxdate(void *, void **);
extern int x1f4_line_lxtime(void *, void **);
extern int x1f4_link_lxdate(void *, void **);
extern int x1f4_link_lxtime(void *, void **);
extern int x1f4_slip_lxtime(void *, void **);

#endif				/* __LIBx1f4_lxdate_H__ */
